--	GameState.lua - Main script run by LuaGameState.cpp.
--  ----------------------------------------------------------------------------
--	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
--	apps.
--	Copyright (C) 2013  Niall Moody
--	
--	This program is free software: you can redistribute it and/or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation, either version 3 of the License, or
--	(at your option) any later version.
--
--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.
--
--	You should have received a copy of the GNU General Public License
--	along with this program.  If not, see <http://www.gnu.org/licenses/>.
--	----------------------------------------------------------------------------

--------------------------------------------------------------------------------
--	The images we draw on screen, indexed by the corresponding key used to
--	trigger them.
images = {}

--	The index of the current image.
imageIndex = "a"
--	The possible key inputs.
keyInputs = {"a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k",
			 "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v",
			"w", "x", "y", "z", "["}

--	The key corresponding to the kick synth.
kickKey = "o"
--	The key corresponding to the clap synth.
clapKey = "p"
--	The key corresponding to the hat synth.
hatKey = "["
--	The keys corresponding to the lo synth.
loKeys = {"z", "x", "c", "v", "b", "n", "m", "l"}
--	The keys corresponding to the hi synth.
hiKeys = {"a", "s", "d", "f", "g", "h", "j", "k"}
--	The keys corresponding to the pad synth.
padKeys = {"q", "w", "e", "r", "t", "y", "u", "i"}

--	Our copy of the size of the drawing area.
drawingSize = TwoFloats()

--	The current background colour.
backgroundColour = ThreeFloats()
--	The current foreground colour.
foregroundColour = ThreeFloats()

--	The countdown images.
countdownImages = {"Countdown3", "Countdown2", "Countdown1", "DANCETIME"}
--	Used to display the initial countdown.
countdown = 0

--	The current step.
currentStep = 0
--	The current pad step.
padStep = 0
--	The contents of the kick steps.
kickSteps = {1, -1, -1, -1, 1, -1, -1, -1, 1, -1, -1, -1, 1, -1, -1, -1}
--	The contents of the clap steps.
clapSteps = {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}
--	The contents of the hat steps.
hatSteps = {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}
--	The contents of the lo steps.
loSteps = {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}
--	The contents of the hi steps.
hiSteps = {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}
--	The contents of the pad steps.
padSteps = {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}
--	The contents of the dancing guy steps.
imageSteps = {"a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a"}
--	The grid size steps.
gridSteps = {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}

--	The tempo the sequencer's running at.
tempo = 120.0
--	Whether distortion is currently on or off.
distortion = false
--	Used to stretch the screen when distortion is on.
squeeze = TwoFloats()

--------------------------------------------------------------------------------
--	Called when the state is first initialised.
function initialise(drawSize)
	local everyImageCombo = {}
	local index = 1

	drawingSize = drawSize

	for i=1,10 do
		for j=1,10 do
			for k=1,10 do
				everyImageCombo[index] = {}
				everyImageCombo[index][1] = "Body" .. i-1
				everyImageCombo[index][2] = "Arms" .. j-1
				everyImageCombo[index][3] = "Legs" .. k-1
	
				index = index + 1
			end
		end
	end

	for i=1,#keyInputs do
		index = Random.getGlobal():getRandomInt(#everyImageCombo-1)+1

		images[keyInputs[i]] = {}
		images[keyInputs[i]][1] = everyImageCombo[index][1]
		images[keyInputs[i]][2] = everyImageCombo[index][2]
		images[keyInputs[i]][3] = everyImageCombo[index][3]
	
		table.remove(everyImageCombo, index)
	end

	updateColours()
end

--------------------------------------------------------------------------------
--	Used to update game logic etc.
function update(sound)
	squeeze.x = squeeze.x + 0.01
	if (squeeze.x > 1.0) then
		squeeze.x = 0.0
	end

	squeeze.y = squeeze.y + 0.013
	if (squeeze.y > 1.0) then
		squeeze.y = 0.0
	end
end

--------------------------------------------------------------------------------
--	Draws the game.
function draw(d)
	--currentImage = imageSteps[currentStep]
	currentImage = {}
	currentImage[1] = images[imageSteps[currentStep+1]][1]
	currentImage[2] = images[imageSteps[currentStep+1]][2]
	currentImage[3] = images[imageSteps[currentStep+1]][3]

	d:fillRect(TwoFloats(0.0),
			   drawingSize,
			   backgroundColour,
			   1.0)

	if distortion then
		local scale = TwoFloats((math.sin(squeeze.x*2.0*math.pi)*0.25)+1.25,
								(math.sin(squeeze.y*2.0*math.pi)*0.25)+1.25)

		d:translate(TwoFloats((drawingSize*0.5) - (drawingSize*0.5) * scale))
		d:scale(scale)
	end

	if (countdown < 4) then
		if (countdown > 0) then
			d:drawImage(countdownImages[countdown],
						(drawingSize*0.5)-256.0,
						TwoFloats(1.0),
						foregroundColour,
						1.0)
		end
	elseif (countdown == 4) then
		d:drawImage(countdownImages[countdown],
					TwoFloats((drawingSize.x*0.5)-512.0, (drawingSize.y*0.5)-256.0),
					TwoFloats(1.0),
					foregroundColour,
					1.0)
	elseif (gridSteps[currentStep+1] == 1) then
		for i=1,3 do
			--d:drawImage(images[imageIndex][i],
			d:drawImage(currentImage[i],
						(drawingSize*0.5)-256.0,
						TwoFloats(1.0),
						foregroundColour,
						1.0)
		end
	else
		local gridSize = gridSteps[currentStep+1]
		local cellSize = drawingSize.x/gridSize
		local yStart = (drawingSize.y*0.5) - (drawingSize.x*0.5)
		
		for y=0,gridSize-1 do
			if (((yStart + (y*cellSize) + cellSize) > 0) and (yStart < drawingSize.y)) then
				for x=0,gridSize-1 do
					if (((y+x)%2) > 0) then
						d:fillRect(TwoFloats(x*cellSize, yStart + (y*cellSize)),
								   TwoFloats(cellSize),
								   foregroundColour,
								   1.0)

						for i=1,3 do
							--d:drawImage(images[imageIndex][i],
							d:drawImage(currentImage[i],
										(TwoFloats(x*cellSize, yStart + (y*cellSize)))+(cellSize*0.5)-(256.0/(gridSize*0.5)),
										TwoFloats(1.0/(gridSize*0.5)),
										backgroundColour,
										1.0)
						end
					else
						for i=1,3 do
							--d:drawImage(images[imageIndex][i],
							d:drawImage(currentImage[i],
										(TwoFloats(x*cellSize, yStart + (y*cellSize)))+(cellSize*0.5)-(256.0/(gridSize*0.5)),
										TwoFloats(1.0/(gridSize*0.5)),
										foregroundColour,
										1.0)
						end
					end
				end
			end
		end
	end
end

--------------------------------------------------------------------------------
--	Called when the user depresses a key or a joystick button.
function keyDown(action, keyPressed, keyModifier, sound)
	--Update dancing guy.
	for i=1,#keyInputs do
		if (keyPressed == keyInputs[i]) then
			imageIndex = keyPressed
			imageSteps[currentStep] = imageIndex
			break
		end
	end

	--Add/subtract notes to the sequencer.
	step = currentStep
	if (keyPressed == kickKey) then
		if (kickSteps[step+1] > -1) then
			sound:setStep(0, step, -1)
			kickSteps[step+1] = -1
		else
			sound:setStep(0, step, 1)
			kickSteps[step+1] = 1
		end
	elseif (keyPressed == clapKey) then
		if (clapSteps[step+1] > -1) then
			sound:setStep(1, step, -1)
			clapSteps[step+1] = -1
		else
			sound:setStep(1, step, 1)
			clapSteps[step+1] = 1
		end
	elseif (keyPressed == hatKey) then
		if (hatSteps[step+1] > -1) then
			sound:setStep(2, step, -1)
			hatSteps[step+1] = -1
		else
			sound:setStep(2, step, 1)
			hatSteps[step+1] = 1
		end
	end
	for i=1,#loKeys do
		if (keyPressed == loKeys[i]) then
			if (loSteps[step+1] == (i-1)) then
				sound:setStep(3, step, -1)
				loSteps[step+1] = -1
			else
				sound:setStep(3, step, i-1)
				loSteps[step+1] = i-1
			end
			break
		end
	end
	for i=1,#hiKeys do
		if (keyPressed == hiKeys[i]) then
			if (hiSteps[step+1] == (i-1)) then
				sound:setStep(4, step, -1)
				hiSteps[step+1] = -1
			else
				sound:setStep(4, step, i-1)
				hiSteps[step+1] = i-1
			end
			break
		end
	end
	for i=1,#padKeys do
		step = padStep
		if (keyPressed == padKeys[i]) then
			if (padSteps[step+1] == (i-1)) then
				sound:setStep(5, step, -1)
				padSteps[step+1] = -1
			else
				sound:setStep(5, step, i-1)
				padSteps[step+1] = i-1
			end
			break
		end
	end

	--Change grid size.
	if (keyPressed == "1") then
		gridSteps[currentStep] = 1
		for i=currentStep,(currentStep+4) do
			if (i > 15) then
				break
			end
			gridSteps[i] = 1
		end
	elseif (keyPressed == "2") then
		gridSteps[currentStep] = 2
		for i=currentStep,(currentStep+4) do
			if (i > 15) then
				break
			end
			gridSteps[i] = 2
		end
	elseif (keyPressed == "3") then
		gridSteps[currentStep] = 3
		for i=currentStep,(currentStep+4) do
			if (i > 15) then
				break
			end
			gridSteps[i] = 3
		end
	elseif (keyPressed == "4") then
		gridSteps[currentStep] = 4
		for i=currentStep,(currentStep+4) do
			if (i > 15) then
				break
			end
			gridSteps[i] = 4
		end
	elseif (keyPressed == "5") then
		gridSteps[currentStep] = 5
		for i=currentStep,(currentStep+4) do
			if (i > 15) then
				break
			end
			gridSteps[i] = 5
		end
	elseif (keyPressed == "6") then
		gridSteps[currentStep] = 6
		for i=currentStep,(currentStep+4) do
			if (i > 15) then
				break
			end
			gridSteps[i] = 6
		end
	elseif (keyPressed == "7") then
		gridSteps[currentStep] = 7
		for i=currentStep,(currentStep+4) do
			if (i > 15) then
				break
			end
			gridSteps[i] = 7
		end
	elseif (keyPressed == "8") then
		gridSteps[currentStep] = 8
		for i=currentStep,(currentStep+4) do
			if (i > 15) then
				break
			end
			gridSteps[i] = 8
		end
	elseif (keyPressed == "9") then
		gridSteps[currentStep] = 9
		for i=currentStep,(currentStep+4) do
			if (i > 15) then
				break
			end
			gridSteps[i] = 9
		end
	elseif (keyPressed == "0") then
		gridSteps[currentStep] = 10
		for i=currentStep,(currentStep+4) do
			if (i > 15) then
				break
			end
			gridSteps[i] = 10
		end
	end

	--Change tempo.
	if (keyPressed == "-") then
		tempo = tempo * 0.5
		sound:setTempo(tempo)
	elseif (keyPressed == "=") then
		if (tempo < 240.0) then
			tempo = tempo * 2.0
			sound:setTempo(tempo)
		end
	end

	--Mute the indexed sequence.
	if (keyPressed == ",") then
		if (keyModifier == 2) then
			for i=0,15 do
				sound:setStep(3, i, -1)
			end
		else
			sound:mute(3, true)
		end
	elseif (keyPressed == ".") then
		if (keyModifier == 2) then
			for i=0,15 do
				sound:setStep(4, i, -1)
			end
		else
			sound:mute(4, true)
		end
	elseif (keyPressed == "/") then
		if (keyModifier == 2) then
			for i=0,15 do
				sound:setStep(5, i, -1)
			end
		else
			sound:mute(5, true)
		end
	elseif (keyPressed == ";") then
		if (keyModifier == 2) then
			for i=0,15 do
				sound:setStep(0, i, -1)
			end
		else
			sound:mute(0, true)
		end
	elseif (keyPressed == "'") then
		if (keyModifier == 2) then
			for i=0,15 do
				sound:setStep(1, i, -1)
			end
		else
			sound:mute(1, true)
		end
	elseif (keyPressed == "#") or (keyPressed == "\\") then
		if (keyModifier == 2) then
			for i=0,15 do
				sound:setStep(2, i, -1)
			end
		else
			sound:mute(2, true)
		end
	elseif (keyPressed == "Space") then
		for i=1,5 do
			sound:mute(i, true)
		end
	elseif (keyPressed == "Delete") or (keyPressed == "Backspace") then
		for i=0,5 do
			for j=0,15 do
				sound:setStep(i, j, -1)
			end
		end

		for i=0,15 do
			gridSteps[i] = 1
		end
	end

	--Turn distortion on/off.
	if (keyPressed == "Return") then
		distortion = not distortion
		sound:setDistortion(distortion)
	end

	--Select a pitch set.
	if (keyPressed == "NumPad0") then
		sound:selectPitches(0)
	elseif (keyPressed == "NumPad1") then
		sound:selectPitches(1)
	elseif (keyPressed == "NumPad2") then
		sound:selectPitches(2)
	elseif (keyPressed == "NumPad3") then
		sound:selectPitches(3)
	elseif (keyPressed == "NumPad4") then
		sound:selectPitches(4)
	elseif (keyPressed == "NumPad5") then
		sound:selectPitches(5)
	elseif (keyPressed == "NumPad6") then
		sound:selectPitches(6)
	elseif (keyPressed == "NumPad7") then
		sound:selectPitches(7)
	elseif (keyPressed == "NumPad8") then
		sound:selectPitches(8)
	elseif (keyPressed == "NumPad9") then
		sound:selectPitches(9)
	end
end

--------------------------------------------------------------------------------
--	Called when the user releases a key or a joystick button.
function keyUp(action, keyPressed, keyModifier, sound)
	--Unmute the indexed sequence.
	if (keyPressed == ",") then
		sound:mute(3, false)
	elseif (keyPressed == ".") then
		sound:mute(4, false)
	elseif (keyPressed == "/") then
		sound:mute(5, false)
	elseif (keyPressed == ";") then
		sound:mute(0, false)
	elseif (keyPressed == "'") then
		sound:mute(1, false)
	elseif (keyPressed == "#") then
		sound:mute(2, false)
	elseif (keyPressed == "Space") then
		for i=1,5 do
			sound:mute(i, false)
		end
	end
end

--------------------------------------------------------------------------------
--	Called when the user depresses a mouse button.
function mouseDown(button, pos)
	
end

--------------------------------------------------------------------------------
--	Called when the user releases a mouse button.
function mouseUp(button, pos)

end

--------------------------------------------------------------------------------
--	Called when the user moves the mouse.
function mouseMove(pos)
	
end

--------------------------------------------------------------------------------
--	Called when the user moves the mousewheel.
function mouseWheel(dir)

end

--------------------------------------------------------------------------------
--	Called when the user moves a joystick.
function joyMove(action, value, joystick, axis)

end

--------------------------------------------------------------------------------
--	Called each time the sequencer advances a step.
function nextStepCallback(step)
	if (imageSteps[step] ~= imageSteps[currentStep]) then
		updateColours()
	end

	currentStep = step
	if ((step%4) == 0) then
		padStep = padStep + 1
		if (padStep > 15) then
			padStep = 0
		end
	end

	if (countdown == 0) then
		if (step == 0) then
			countdown = countdown + 1
			updateColours()
		end
	elseif (countdown < 5) then
		if ((step%4) == 0) then
			countdown = countdown + 1
			updateColours()
		end
	end
end

--------------------------------------------------------------------------------
--	Helper method. Updates colours.
function updateColours()
	backgroundColour.x = Random.getGlobal():getRandomFloat()
	backgroundColour.y = 1.0
	backgroundColour.z = 0.5

	foregroundColour = ThreeFloats(backgroundColour)
	foregroundColour.x = foregroundColour.x + 0.33
	if (foregroundColour.x > 1.0) then
		foregroundColour.x = foregroundColour.x - 1.0
	end

	backgroundColour = hslToRgb(backgroundColour)
	foregroundColour = hslToRgb(foregroundColour)
end
