//	DANCETIMESounder.cpp - The game's sounder.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

#include "DANCETIMESounder.h"
#include "Random.h"

#include "PastaPickup.inl"
#include "PastaThing.inl"
#include "PastaSlow.inl"
#include "PastaMine.inl"
#include "PastaPew.inl"
#include "PastaLo.inl"
#include "PastaHi.inl"

#include <cassert>
#include <sstream>
#include <string>

//------------------------------------------------------------------------------
DANCETIMESounder::DANCETIMESounder():
padStep(0),
musicPlaying(true),
distortion(false),
fadeOut(1.0f),
callback(0)
{
	int i;

	amplitudes[0] = 1.0f;
	amplitudes[1] = 1.0f;
	amplitudes[2] = 1.0f;

	for(i=0;i<16;++i)
	{
		kickTriggers[i] = false;
		clapTriggers[i] = false;
		hatTriggers[i] = false;

		loTriggers[i] = -1;
		hiTriggers[i] = -1;
		padTriggers[i] = -1;
	}
	for(i=0;i<6;++i)
		muted[i] = false;

	kickTriggers[0] = true;
	kickTriggers[4] = true;
	kickTriggers[8] = true;
	kickTriggers[12] = true;
	/*clapTriggers[4] = true;
	clapTriggers[12] = true;
	hatTriggers[2] = true;
	hatTriggers[6] = true;
	hatTriggers[10] = true;
	hatTriggers[14] = true;

	loTriggers[0] = 7;
	loTriggers[2] = 6;
	loTriggers[8] = 7;
	loTriggers[10] = 6;
	loTriggers[14] = 5;

	hiTriggers[1] = 0;
	hiTriggers[3] = 1;
	hiTriggers[4] = 0;
	hiTriggers[9] = 0;
	hiTriggers[11] = 1;
	hiTriggers[12] = 2;
	hiTriggers[14] = 3;

	padTriggers[0] = 4;
	padTriggers[4] = 0;
	padTriggers[8] = 3;
	padTriggers[12] = 1;*/

	hiSynth.loadParameters(PastaHi::numParameters, PastaHi::parameters);
	hiDelay.setDelay(0.125f);
	hiDelay.setFeedback(0.75f);

	loSynth.loadParameters(PastaLo::numParameters, PastaLo::parameters);
	loDelay.setDelay(0.25f);
	loDelay.setFeedback(0.75f);

	slowSynth.loadParameters(PastaSlow::numParameters, PastaSlow::parameters);
	slowDelay.setDelay(0.33f);
	slowDelay.setFeedback(0.75f);

	mineSynth.loadParameters(PastaMine::numParameters, PastaMine::parameters);
	mineDelay.setDelay(0.125f);
	mineDelay.setFeedback(0.5f);

	pickupSynth.loadParameters(PastaThing::numParameters, PastaThing::parameters);
	pickupDelay.setDelay(0.125f);
	pickupDelay.setFeedback(0.5f);

	pewSynth.loadParameters(PastaPew::numParameters, PastaPew::parameters);
	pewDelay.setDelay(0.125f);
	pewDelay.setFeedback(0.75f);

	percDelay.setDelay(0.125f);
	percDelay.setFeedback(0.5f);

	loadPitches();

	cursor.registerCallback(this);
}

//------------------------------------------------------------------------------
DANCETIMESounder::~DANCETIMESounder()
{

}

//------------------------------------------------------------------------------
void DANCETIMESounder::getAudio(float *input, float *output, int numSamples)
{
	int i;
	float loAmp;

	assert(numSamples <= (BufferSize/2));

	cursor.audioBlock(numSamples);

	for(i=0;i<(numSamples*2);++i)
	{
		loBuffer[i] = 0.0f;
		slowBuffer[i] = 0.0f;
		mineBuffer[i] = 0.0f;
		pickupBuffer[i] = 0.0f;
		pewBuffer[i] = 0.0f;
		percBuffer[i] = 0.0f;

		output[i] = 0.0f;
	}

	hiSynth.getAudio(output, numSamples);
	hiDelay.getAudio(output, numSamples);

	loSynth.getAudio(loBuffer, numSamples);
	//loDelay.getAudio(loBuffer, numSamples);

	slowSynth.getAudio(slowBuffer, numSamples);
	slowDelay.getAudio(slowBuffer, numSamples);

	mineSynth.getAudio(mineBuffer, numSamples);
	mineDelay.getAudio(mineBuffer, numSamples);

	pickupSynth.getAudio(pickupBuffer, numSamples);
	pickupDelay.getAudio(pickupBuffer, numSamples);

	pewSynth.getAudio(pewBuffer, numSamples);
	pewDelay.getAudio(pewBuffer, numSamples);

	kick.getAudio(percBuffer, numSamples);
	if(clap.getActive())
		clap.getAudio(percBuffer, numSamples);
	if(hat.getActive())
		hat.getAudio(percBuffer, numSamples);
	//percDelay.getAudio(percBuffer, numSamples);

	if(distortion)
		loAmp = 0.5f;
	else
		loAmp = 1.0f;

	for(i=0;i<(numSamples*2);++i)
	{
		output[i] += loBuffer[i] * loAmp;
		output[i] += slowBuffer[i];
		output[i] += mineBuffer[i];
		output[i] += pickupBuffer[i];
		output[i] += pewBuffer[i];
	}

	for(i=0;i<(numSamples*2);++i)
		output[i] += percBuffer[i] * 0.5f;

	if(distortion)
	{
		for(i=0;i<(numSamples*2);++i)
			output[i] = tanhf(output[i] * 24.0f) * 0.2f;
	}

	if(fadeOut < 1.0f)
	{
		for(i=0;i<numSamples;++i)
		{
			output[(i*2)] *= fadeOut;
			output[(i*2)+1] *= fadeOut;

			if(fadeOut > 0.0f)
				fadeOut -= 0.00001f;
		}
	}
}

//------------------------------------------------------------------------------
void DANCETIMESounder::triggerHi(float val)
{
	int index = (int)(val * 7.99f);

	hiSynth.triggerNote(pitches[index][currentPitches]*2.0f, 0);
}

//------------------------------------------------------------------------------
void DANCETIMESounder::triggerLo(float val)
{
	int index = (int)(val * 7.99f);

	loSynth.triggerNote(pitches[index][currentPitches], 0);
}

//------------------------------------------------------------------------------
void DANCETIMESounder::triggerSlow(float val)
{
	int index = (int)(val * 7.99f);

	slowSynth.triggerNote(pitches[index][currentPitches], 0);
}

//------------------------------------------------------------------------------
void DANCETIMESounder::triggerMine(float val)
{
	int index = (int)(val * 7.99f);

	mineSynth.triggerNote(pitches[index][currentPitches], 0);
}

//------------------------------------------------------------------------------
void DANCETIMESounder::triggerPickup(float val)
{
	int index = (int)(val * 7.99f);

	pickupSynth.triggerNote(pitches[index][currentPitches], 0);
}

//------------------------------------------------------------------------------
void DANCETIMESounder::triggerPew(float val)
{
	int index = (int)(val * 7.99f);

	pewSynth.triggerNote(pitches[index][currentPitches], 0);
}

//------------------------------------------------------------------------------
void DANCETIMESounder::triggerKick(int sample)
{
	kick.triggerNote(440.0f, sample, 1.0f);
}

//------------------------------------------------------------------------------
void DANCETIMESounder::triggerClap(int sample)
{
	clap.setActive(true, sample, false);
}

//------------------------------------------------------------------------------
void DANCETIMESounder::triggerHat(int sample)
{
	hat.setActive(true, sample, false);
}

//------------------------------------------------------------------------------
void DANCETIMESounder::updateNotes(int *val)
{
	
}

//------------------------------------------------------------------------------
void DANCETIMESounder::updateNotesHi(int *val)
{
	
}

//------------------------------------------------------------------------------
void DANCETIMESounder::updateNotesOther(int *val)
{
	
}

//------------------------------------------------------------------------------
void DANCETIMESounder::setMusicPlaying(bool val)
{
	musicPlaying = val;
}

//------------------------------------------------------------------------------
void DANCETIMESounder::resetGhosts()
{
	
}

//------------------------------------------------------------------------------
void DANCETIMESounder::setDistortion(bool val)
{
	distortion = val;
}

//------------------------------------------------------------------------------
void DANCETIMESounder::setNoteAmp(float player, float ghost1, float ghost2)
{
	amplitudes[0] = player;
	amplitudes[1] = ghost1;
	amplitudes[2] = ghost2;
}

//------------------------------------------------------------------------------
void DANCETIMESounder::setStep(int instrument, int step, int value)
{
	switch(instrument)
	{
		case 0:
			kickTriggers[step] = (value > -1);
			if(kickTriggers[step])
				kick.triggerNote(440.0f, 0, 1.0f);
			break;
		case 1:
			clapTriggers[step] = (value > -1);
			if(clapTriggers[step])
				clap.setActive(true, 0, false);
			break;
		case 2:
			hatTriggers[step] = (value > -1);
			if(hatTriggers[step])
				hat.setActive(true, 0, false);
			break;
		case 3:
			loTriggers[step] = value;
			if(value > -1)
				loSynth.triggerNote(pitches[loTriggers[step]][currentPitches], 0);
			break;
		case 4:
			hiTriggers[step] = value;
			if(value > -1)
				hiSynth.triggerNote(pitches[hiTriggers[step]][currentPitches], 0);
			break;
		case 5:
			padTriggers[step] = value;
			if(value > -1)
				slowSynth.triggerNote(pitches[padTriggers[step]][currentPitches], 0);
			break;
	}
}

//------------------------------------------------------------------------------
void DANCETIMESounder::clear()
{
	int i;

	for(i=0;i<16;++i)
	{
		kickTriggers[i] = false;
		clapTriggers[i] = false;
		hatTriggers[i] = false;
		loTriggers[i] = -1;
		hiTriggers[i] = -1;
		padTriggers[i] = -1;
	}

	fadeOut = 0.99999f;
}

//------------------------------------------------------------------------------
void DANCETIMESounder::setTempo(float val)
{
	cursor.setTempo(val);
}

//------------------------------------------------------------------------------
void DANCETIMESounder::mute(int instrument, bool val)
{
	muted[instrument] = val;
}

//------------------------------------------------------------------------------
void DANCETIMESounder::nextStepCallback(int step, int sample)
{
	int tempint;

	if(kickTriggers[step] && !muted[0])
		kick.triggerNote(440.0f, sample, 1.0f);
	if(clapTriggers[step] && !muted[1])
		clap.setActive(true, sample, false);
	if(hatTriggers[step] && !muted[2])
		hat.setActive(true, sample, false);

	if((loTriggers[step] > -1)  && !muted[3])
		loSynth.triggerNote(pitches[loTriggers[step]][currentPitches], sample);
	if((hiTriggers[step] > -1) && !muted[4])
		hiSynth.triggerNote(pitches[hiTriggers[step]][currentPitches], sample);
	if(!(step%4))
	{
		++padStep;
		padStep %= 16;
		if((padTriggers[padStep] > -1) && !muted[5])
			slowSynth.triggerNote(pitches[padTriggers[padStep]][currentPitches], sample);
	}

	if(callback)
		callback->nextStepCallback(step, sample);
}

//------------------------------------------------------------------------------
void DANCETIMESounder::loadPitches()
{
	int i;
	int index;
	FilePath pitchesDir = FilePath::getDataFilesDir().getChild(L"pitches");

	for(i=0;i<pitchesDir.getNumChildren();++i)
	{
		FilePath pitchFile = pitchesDir.getChild(i);

		if(pitchFile.exists())
		{
			if(pitchFile.getExtension() == L"txt")
			{
				UTF8File phil(pitchFile);

				if(!phil.getFail())
				{
					int i;
					float tempf;
					std::wstring tempstr;
					std::wstringstream tempstr2;

					i = 0;
					while(!phil.readLine(tempstr))
					{
						assert(i < 8);

						tempstr2.clear();
						tempstr2.str(L"");
						tempstr2 << tempstr;

						tempstr2 >> tempf;

						pitches[i].push_back(tempf * 0.25f);

						++i;
					}
				}
			}
		}
	}
	currentPitches = Random::getGlobal().getRandomInt(pitches[0].size());
}

//------------------------------------------------------------------------------
void DANCETIMESounder::selectRandomPitches()
{
	int index = Random::getGlobal().getRandomInt(pitches[0].size());

	while(index == currentPitches)
		index = Random::getGlobal().getRandomInt(pitches[0].size());

	currentPitches = index;
}

//------------------------------------------------------------------------------
void DANCETIMESounder::selectPitches(int index)
{
	if(index < pitches[0].size())
		currentPitches = index;
}

//------------------------------------------------------------------------------
void DANCETIMESounder::registerCallback(Callback *c)
{
	callback = c;
}
