//	ClapSynth.cpp - The clap synthesizer.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2010  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "ClapSynth.h"
#include "MiscHelper.h"
#include "Random.h"

#include <cmath>

//------------------------------------------------------------------------------
ClapSynth::ClapSynth():
active(false),
start(0),
clapReset(3),
length(0.0f),
lengthScale(1.0f),
increment(8.0f/44100.0f),
filter(0.0f),
filter2(0.0f),
filterCoeff(0.999f)
{
	moogFilter.init();
	moogFilter.setCutoffAndRes(3500.0f, 0.167f);
}

//------------------------------------------------------------------------------
ClapSynth::~ClapSynth()
{
	
}

//------------------------------------------------------------------------------
void ClapSynth::getAudio(float *buffer, int numSamples)
{
	int i;
	//float amp = 0.15f;
	float tempf, tempf2;
	float expLength;

	for(i=start;i<numSamples;++i)
	{
		//Count down the sound's timer.
		length -= increment;
		tempf = 0.925f;
		if((length < tempf) && (clapReset > 0))
		{
			--clapReset;
			if(clapReset > 0)
				length = 0.9f + (Random::getGlobal().getRandomFloat() * 0.1f);
			else
			{
				length = 6.0f;
				lengthScale = 0.167f;
			}
		}
		if(length <= 0.0f)
		{
			setActive(false, 0, false);
			break;
		}

		//Generate the audio.
		tempf = (Random::getGlobal().getRandomFloat() * 2.0f) - 1.0f;
		expLength = fakeExp(length*lengthScale);
		tempf2 = tanhf(tempf * 3.14159265f * 1.5f) * expLength * expLength * expLength;
		/*tempf = (tempf2 + (filter * filterCoeff));
		filter = tempf2;
		tempf2 = (tempf + (filter2 * filterCoeff));
		filter2 = tempf;
		tempf = tempf2;*/
		tempf = moogFilter.process(tempf2);
		buffer[i*2] += tempf;// * 0.5f;
		buffer[(i*2)+1] += tempf;// * 0.5f;
	}
	start = 0;
}

//------------------------------------------------------------------------------
void ClapSynth::setActive(bool val, int startSample, bool hi)
{
	active = val;
	if(active)
	{
		start = startSample;

		length = 1.0f;
		lengthScale = 1.0f;
		if(hi)
		{
			clapReset = 2;
			filterCoeff = 0.999f;
		}
		else
		{
			clapReset = 4;
			filterCoeff = 0.1f;
		}
	}
}
