//	ClapSynth.h - The clap synthesizer.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2010  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef CLAPSYNTH_H_
#define CLAPSYNTH_H_

#include "Synthesizer.h"
#include "MoogFilter.h"

///	The clap synthesizer.
class ClapSynth : public Synthesizer
{
  public:
	///	Constructor.
	ClapSynth();
	///	Destructor.
	~ClapSynth();

	///	Returns the audio for this synthesizer.
	void getAudio(float *buffer, int numSamples);

	///	Sets whether or not this synthesizer is active.
	void setActive(bool val, int startSample, bool hi);
	///	Returns whether or not this synthesizer is active.
	bool getActive() const {return active;};
  private:
	///	Whether or not the synthesizer is currently active.
	bool active;
	///	Which sample to start on in the next audio block.
	int start;

	///	Used to trigger the characteristic trill(?) sound of the clap.
	int clapReset;
	///	The length of the kick sound (counts down).
	float length;
	///	The amount to scale length by in fakeExp.
	float lengthScale;
	///	The value used to count down length.
	float increment;

	///	The lo-pass filter.
	float filter;
	///	The lo-pass filter.
	float filter2;
	///	The filter coefficient.
	float filterCoeff;

	///	The filter.
	MoogFilter moogFilter;
};

#endif
