//	KickSynth.cpp - The kick drum synthesizer.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2010  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "KickSynth.h"
#include "MiscHelper.h"

#include <cmath>

//------------------------------------------------------------------------------
KickSynth::KickSynth():
active(false),
start(0),
length(0.0f),
increment(8.0f/44100.0f),
fadeIn(1.0f),
x0(1.0f),
x1(0.0f),
hiIndex(0)
{
	hiCoeff[0] = 2.0f*sinf(3.1415926535897932384626433832795f*146.83238f/44100.0f);
	hiCoeff[1] = 2.0f*sinf(3.1415926535897932384626433832795f*110.0f/44100.0f);
	loCoeff = 2.0f*sinf(3.1415926535897932384626433832795f*50.0f/44100.0f);

	coeff = hiCoeff[0];
}

//------------------------------------------------------------------------------
KickSynth::~KickSynth()
{
	
}

//------------------------------------------------------------------------------
void KickSynth::getAudio(float *buffer, int numSamples)
{
	int i;
	float amp;
	float tempf;

	for(i=start;i<numSamples;++i)
	{
		//Count down the sound's timer.
		length -= increment;
		if(length <= 0.0f)
		{
			setActive(false, 0, false);
			break;
		}

		//Generate the audio.
		x0 -= (coeff*x1);
		x1 += (coeff*x0);
		tempf = tanhf(x1 * 3.14159265f * 0.66f);
		if(fadeIn < 1.0f)
			amp = fadeIn * 0.5f;
		else
			amp = fakeLog(length) * 0.5f;
		buffer[i*2] += tempf * amp;
		buffer[(i*2)+1] += tempf * amp;

		coeff = (loCoeff * (1.0f-length)) + (hiCoeff[hiIndex] * length);
		tempf = fabsf(x0)+fabsf(x1);
		if(tempf > 1.0f)
		{
			tempf = 1.0f/tempf;
			x0 *= tempf;
			x1 *= tempf;
		}

		if(fadeIn < 1.0f)
		{
			fadeIn += 0.001f;
			if(fadeIn >= 1.0f)
			{

			}
		}
	}
	start = 0;
}

//------------------------------------------------------------------------------
void KickSynth::setActive(bool val, int startSample, bool hi)
{
	active = val;
	if(active)
	{
		start = startSample;

		if(length > 0.0f)
			fadeIn = fakeLog(length);
		else
			fadeIn = 1.0f;
		length = 1.0f;
		x0 = 1.0f;
		x1 = 0.0f;
		hiIndex = hi ? 1 : 0;
		//coeff = hiCoeff[hiIndex];
	}
}
